unit uCollectionDef;
{$INCLUDE colbuild.inc}

interface

uses
  SysUtils, Classes, Contnrs,
  janXmlParser2,
  RegExpr,
  uFileList;

type
  TCollectionItem = class
    private
    protected
    public
      Name: string;
      FileNode: TFileItem;
    end;

  TCollectionDef = class(TObjectList)
    private
      //fID: integer;
      fTitle: string;
      fXmlNode: TjanXmlNode2;
    protected
      function GetItem(Index: Integer): TCollectionItem;
      procedure SetItem(Index: Integer; AItem: TCollectionItem);
    public
      function Add(AItem: TCollectionItem): Integer;
      function Remove(AItem: TCollectionItem): Integer;
      function IndexOf(AItem: TCollectionItem): Integer; overload;
      function IndexOf(const AName: string): Integer; overload;
      function IndexOf(AFileNode: TFileItem): Integer; overload;
      procedure Insert(Index: Integer; AItem: TCollectionItem);
      //property ID: integer read fID write fID;
      property Title: string read fTitle write fTitle;
      property Items[Index: Integer]: TCollectionItem read GetItem write SetItem; default;
      property XmlNode: TjanXmlNode2 read fXmlNode write fXmlNode;
    end;

  TCollectionList = class;

  TCollectionConfigEvent = procedure (Sender: TCollectionList; const Name, Value: string) of object;

  TCollectionList = class(TObjectList)
    private
      fOnConfig: TCollectionConfigEvent;
    protected
      function GetItem(Index: Integer): TCollectionDef;
      procedure SetItem(Index: Integer; AItem: TCollectionDef);
      procedure DoConfig(const Name, Value: string);
    public
      function Add(AItem: TCollectionDef): Integer;
      function Remove(AItem: TCollectionDef): Integer;
      function IndexOf(AItem: TCollectionDef): Integer; overload;
      function IndexOf(const ATitle: string): Integer; overload;
      procedure Insert(Index: Integer; AItem: TCollectionDef);
      procedure LoadFromFile(const FileName: string);
      procedure LoadFromStream(Stream: TStream);
      procedure BuildFromRegExp(FileList: TFileList; const RegExp, Title: string);
      property Items[Index: Integer]: TCollectionDef read GetItem write SetItem; default;
      property OnConfig: TCollectionConfigEvent read fOnConfig write fOnConfig;
    end;

implementation

{ TCollectionDef }

function TCollectionDef.Add(AItem: TCollectionItem): Integer;
begin
  Result := inherited Add(AItem);
end;

function TCollectionDef.GetItem(Index: Integer): TCollectionItem;
begin
  Result := inherited Items[Index] as TCollectionItem;
end;

function TCollectionDef.IndexOf(AItem: TCollectionItem): Integer;
begin
  Result := inherited IndexOf(AItem);
end;

function TCollectionDef.IndexOf(const AName: string): Integer;
begin
  Result := Pred(Count);
  while Result >= 0 do
    if AnsiCompareText(Items[Result].Name, AName) = 0 then
      Break
    else
      Dec(Result);
end;

function TCollectionDef.IndexOf(AFileNode: TFileItem): Integer;
begin
  Result := Pred(Count);
  while Result >= 0 do
    if Items[Result].FileNode = AFileNode then
      Break
    else
      Dec(Result);
end;

procedure TCollectionDef.Insert(Index: Integer; AItem: TCollectionItem);
begin
  inherited Insert(Index, AItem);
end;

function TCollectionDef.Remove(AItem: TCollectionItem): Integer;
begin
  Result := inherited Remove(AItem);
end;

procedure TCollectionDef.SetItem(Index: Integer; AItem: TCollectionItem);
begin
  inherited Items[Index] := AItem;
end;

{ TCollectionList }

function TCollectionList.Add(AItem: TCollectionDef): Integer;
begin
  Result := inherited Add(AItem);
end;

procedure TCollectionList.BuildFromRegExp(FileList: TFileList; const RegExp, Title: string);
var RE: TRegExpr;
    i, j: integer;
    FN, Playlist: string;
    Def: TCollectionDef;
    Item: TCollectionItem;
begin
  if RegExp = '' then
    Exit;
  if FileList.Count <= 0 then
    Exit;
  RE := TRegExpr.Create;
  try
    RE.ModifierI := True;
    RE.Expression := RegExp;
    for i := 0 to Pred(FileList.Count) do
      begin
      FN := FileList[i].Name;
      if RE.Exec(FN) then
        begin
        Playlist := RE.Substitute(Title);
        j := IndexOf(PlayList);
        if j >= 0 then
          Def := Items[j]
        else
          begin
          Def := TCollectionDef.Create;
          Def.Title := Playlist;
          Add(Def);
          end;
        j := Def.IndexOf(FileList[i]);
        if j < 0 then
          begin
          Item := TCollectionItem.Create;
          Item.Name := FileList[i].Name;
          Item.FileNode := FileList[i];
          Def.Add(Item);
          end;
        end;
      end;
  finally
    RE.Free;
    end;
end;

procedure TCollectionList.DoConfig(const Name, Value: string);
begin
  if Assigned(fOnConfig) then
    fOnConfig(Self, Name, Value);
end;

function TCollectionList.GetItem(Index: Integer): TCollectionDef;
begin
  Result := inherited Items[Index] as TCollectionDef;
end;

function TCollectionList.IndexOf(AItem: TCollectionDef): Integer;
begin
  Result := inherited IndexOf(AItem);
end;

function TCollectionList.IndexOf(const ATitle: string): Integer;
begin
  Result := Pred(Count);
  while Result >= 0 do
    if Items[Result].Title = ATitle then
      Break
    else
      Dec(Result);
end;

procedure TCollectionList.Insert(Index: Integer; AItem: TCollectionDef);
begin
  inherited Insert(Index, AItem);
end;

procedure TCollectionList.LoadFromFile(const FileName: string);
var Strm: TFileStream;
begin
  Strm := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Strm);
  finally
    Strm.Free;
    end;
end;

procedure TCollectionList.LoadFromStream(Stream: TStream);
var L: TStringList;
    i, j, n: integer;
    s, Name, Value: string;
    Def: TCollectionDef;
    Item: TCollectionItem;
begin
  L := TStringList.Create;
  try
    L.LoadFromStream(Stream);
    Def := nil;
    for i := 0 to Pred(L.Count) do
      begin
      s := Trim(L[i]);
      n := Length(s);
      if s <> '' then
        begin // New playlist
        if (s[1] = '[') and (s[n] = ']') then
          begin
          Def := TCollectionDef.Create;
          Def.Title := Trim(Copy(s, 2, n-2));
          Add(Def);
          end
        else if Def <> nil then // Playlist item
          begin
          Item := TCollectionItem.Create;
          Item.Name := s;
          Def.Add(Item);
          end
        else // Playlist collection option
          begin
          j := Pos('=', s);
          if j > 0 then
            begin
            Name := Trim(Copy(s, 1, Pred(j)));
            Value := Trim(Copy(s, Succ(j), MaxInt));
            end
          else
            begin
            Name := s;
            Value := '';
            end;
          DoConfig(Name, Value);
          end;
        end;
      end;
  finally
    L.Free;
    end;
end;

function TCollectionList.Remove(AItem: TCollectionDef): Integer;
begin
  Result := inherited Remove(AItem);
end;

procedure TCollectionList.SetItem(Index: Integer; AItem: TCollectionDef);
begin
  inherited Items[Index] := AItem;
end;

end.
