unit uFileList;
{$INCLUDE colbuild.inc}

interface

uses
  Windows, SysUtils, Classes, Contnrs,
  janXmlParser2,
  RegExpr;

type
  TFileItem = class
    private
    protected
    public
      Directory, Name: string;
      Size: integer;
      Date, CreationDate: TDateTime;
      XmlNode: TjanXmlNode2;
    end;

  TFileList = class(TObjectList)
    private
    protected
      function GetItem(Index: Integer): TFileItem;
      procedure SetItem(Index: Integer; AItem: TFileItem);
      procedure DoLoadFromDir(const Root, Dir: string; Recursive: boolean; const RegExp: string);
    public
      function Add(AItem: TFileItem): Integer;
      function Remove(AItem: TFileItem): Integer;
      function IndexOf(AItem: TFileItem): Integer; overload;
      function IndexOf(const AName: string): Integer; overload;
      procedure Sort;
      procedure Insert(Index: Integer; AItem: TFileItem);
      procedure LoadFromDir(const Dir: string; Recursive: boolean = True; const RegExp: string = ''); // Dir must end with '\'
      property Items[Index: Integer]: TFileItem read GetItem write SetItem; default;
    end;

implementation

{ TFileList }

function TFileList.Add(AItem: TFileItem): Integer;
begin
  Result := inherited Add(AItem);
end;

function TFileList.GetItem(Index: Integer): TFileItem;
begin
  Result := inherited Items[Index] as TFileItem;
end;

function TFileList.IndexOf(AItem: TFileItem): Integer;
begin
  Result := inherited IndexOf(AItem);
end;

function TFileList.IndexOf(const AName: string): Integer;
begin
  Result := Pred(Count);
  while Result >= 0 do
    if AnsiCompareText(Items[Result].Name, AName) = 0 then
      Break
    else
      Dec(Result);
end;

procedure TFileList.Insert(Index: Integer; AItem: TFileItem);
begin
  inherited Insert(Index, AItem);
end;

procedure TFileList.DoLoadFromDir(const Root, Dir: string; Recursive: boolean; const RegExp: string);
var SR: TSearchRec;
    OK: integer;
    s: string;
    i: integer;
    RE: TRegExpr;
    CTime: TFileTime;
    DosTime: integer;
begin
  OK := FindFirst(Root + Dir + '*.*', faAnyFile, SR);
  if OK = 0 then
    try
      RE := nil;
      try
        if RegExp <> '' then
          begin
          RE := TRegExpr.Create;
          RE.ModifierI := True;
          RE.Expression := RegExp;
          end;
        repeat
          if Longbool(SR.Attr and faVolumeID) then
            begin
            end
          else if Longbool(SR.Attr and faDirectory) then
            begin
            if Recursive and (SR.Name <> '.') and (SR.Name <> '..') then
              DoLoadFromDir(Root, Dir + SR.Name + '\', Recursive, RegExp);
            end
          else
            begin
            s := Dir + SR.Name;
            if (RE = nil) or (RE.Exec(s)) then
              begin
              i := IndexOf(s);
              if i < 0 then
                i := Add(TFileItem.Create);
              with Items[i] do
                begin
                Directory := Root;
                Name := s;
                Size := SR.Size;
                Date := FileDateToDateTime(SR.Time);
                FileTimeToLocalFileTime(SR.FindData.ftCreationTime, CTime);
                FileTimeToDosDateTime(CTime, LongRec(DosTime).Hi, LongRec(DosTime).Lo);
                CreationDate := FileDateToDateTime(DosTime);
                end;
              end;
            end;
          OK := FindNext(SR);        
        until OK <> 0;
      finally
        RE.Free;
        end;
    finally
      FindClose(SR);
      end;
end;

procedure TFileList.LoadFromDir(const Dir: string; Recursive: boolean; const RegExp: string);
begin
  DoLoadFromDir(Dir, '', Recursive, RegExp);
  Sort;
end;

function TFileList.Remove(AItem: TFileItem): Integer;
begin
  Result := inherited Remove(AItem);
end;

procedure TFileList.SetItem(Index: Integer; AItem: TFileItem);
begin
  inherited Items[Index] := AItem;
end;

procedure TFileList.Sort;

  {$IFDEF FPC}
  function FileListCompare(Item1, Item2: pointer): integer;
    begin
      // For some strange reason, FreePascal keeps putting nonsensical values
      // into Item2 (it's not even an object!)
      Result := 0;
      //Result := AnsiCompareText(TFileItem(Item1).Name, TFileItem(Item2).Name);
    end;
  {$ELSE}
  function FileListCompare(Item1, Item2: TFileItem): integer;
    begin
      Result := AnsiCompareText(Item1.Name, Item2.Name);
    end;
  {$ENDIF}

begin
  inherited Sort(@FileListCompare);
end;

end.
