unit uFunc;
{$INCLUDE colbuild.inc}

interface

uses
  SysUtils;

function IsAbsolutePath(const Path: string): boolean;
function DirectoryExists(const Path: string): boolean;
function WideToUTF8(const Value: WideString): string;
function AnsiToUTF8(const Value: string): string;
function OemToUTF8(const Value: string): string;

implementation

uses
  Windows;

function IsAbsolutePath(const Path: string): boolean;
var Drive, Dir: string;
begin
  Drive := ExtractFileDrive(Path);
  Dir := ExtractFilePath(Path);
  Result := (Drive <> '') or ((Dir <> '') and (Dir[1] = '\'));
end;

function DirectoryExists(const Path: string): boolean;
var SR: TSearchRec;
    OK: integer;
begin
  Result := False;
  OK := FindFirst(ExcludeTrailingBackslash(Path), faDirectory, SR);
  if OK = 0 then
    begin
    Result := Longbool(SR.Attr and faDirectory);
    SysUtils.FindClose(SR);
    end;
end;

function WideToUTF8(const Value: WideString): string;
var n: integer;
begin
  n := 4*Length(Value);
  SetLength(Result, n);
  n := WideCharToMultiByte(CP_UTF8, 0, PWideChar(Value), Length(Value), @Result[1], n, nil, nil);
  if n = 0 then
    Result := Value
  else
    SetLength(Result, n);
end;

function AnsiToUTF8(const Value: string): string;
var WideValue: WideString;
    n: integer;
begin
  WideValue := Value;
  n := WideCharToMultiByte(CP_UTF8, 0, PWideChar(WideValue), Length(WideValue), nil, 0, nil, nil);
  if n > 0 then
    Result := WideToUTF8(WideValue)
  else
    Result := Value;
end;

function OemToUTF8(const Value: string): string;
begin
  Result := Value;
  if OemToChar(@(Value[1]), @(Result[1])) then
    Result := AnsiToUTF8(Result)
  else
    Result := Value;
end;

end.
